@@ -1,7 +1,6 @@
 #include<iostream>
 
-int main() {
+int main(void) {
   std::cout << "Hello World" << std::endl;
   return 0;
 }
-
