@@ -49,16 +49,18 @@ def get_entries() -> T.List[T.Tuple[int, str]]:
             numstr = int(number)
         except ValueError:
             raise SystemExit(f'Dir name {e} does not start with a number.')
+        if 'includedirxyz' in e:
+            continue
         entries.append((numstr, rest))
     entries.sort()
     return entries
 
 def replace_source(sourcefile: str, replacements: T.List[T.Tuple[str, str]]) -> None:
-    with open(sourcefile) as f:
+    with open(sourcefile, encoding='utf-8') as f:
         contents = f.read()
     for old_name, new_name in replacements:
         contents = contents.replace(old_name, new_name)
-    with open(sourcefile, 'w') as f:
+    with open(sourcefile, 'w', encoding='utf-8') as f:
         f.write(contents)
 
 def condense(dirname: str) -> None:
@@ -81,6 +83,8 @@ def condense(dirname: str) -> None:
     os.chdir(curdir)
     replace_source('run_unittests.py', replacements)
     replace_source('run_project_tests.py', replacements)
+    for f in glob('unittests/*.py'):
+        replace_source(f, replacements)
 
 if __name__ == '__main__':
     if len(sys.argv) != 1:
